	include	p16f628.inc
	include data.inc

	global	readADC, init_ADC
	
#define	ADC_CLK	PORTB,2
#define	ADC_DATA PORTB,1
#define	ADC_DATA_TRIS	TRISB,1
#define	ADC_CS	PORTB,5	

	;; code to operate MC3204 serial ADC
	;;
	;; I/O pins used:
	;; RB5 - chip select (out)
	;; RB2 - serial clock (out)
	;; RB1 - serial data (in/out)
	;;
	;; routine is called with setup word to write to chip loaded in
	;; location T0.
	;;
	;; Uses location LOOPCOUNT

	;;  Result returned in T0, T1
	;;
	;; Calls:
	;;	ADCdelay
	;;	clockpulse
	;;
	;; Assumes data bank 0 is currently selected and returns that way

	code


	;;
	;; Setup PORTB pins used by the ADC.
	;;
	
init_ADC:
	bcf	ADC_CLK		; make clock out low
	bsf	ADC_CS		; disable chip select
	bsf	STATUS,RP0	; data page 1
	bcf	TRISB,5		; make chip select pin an output
	bcf	TRISB,2		; make clock pin an output
	bcf	STATUS,RP0	; back to data page 0
	return
	
	;;
	;; ADCdelay
	;;
	;; Waste a little time
	;; 
ADCdelay:
	nop
	nop
	nop
	nop
	return

	;;
	;; clockpulse
	;;
	;; Pulse the ADC clock line
	;;
clockpulse:
	bsf	ADC_CLK		; set clock high
	call	ADCdelay	; pause
	bcf	ADC_CLK		; set clock low
	call	ADCdelay	; pause
	retlw	0		; done

	;; 
readADC:
	movf	T0,W		; make a copy of the command byte
	movwf	ADCcommand
startconversion:
	movf	ADCcommand,W	; get command byte
	movwf	T0

	bsf	STATUS, RP0	; select data bank 1
	bcf	ADC_DATA_TRIS	; change RB1 to output
	bcf	STATUS,RP0	; return to data bank 0
	bcf	ADC_CLK		; make sure clock low
	bcf	ADC_CS		; assert chip select
	movlw	6
	movwf	LOOPCOUNT
ADCsetuploop:
	bcf	ADC_DATA	; output a zero for now
	btfsc	T0,0		; check value to output
	bsf	ADC_DATA	; it was a one
	call	ADCdelay	; wait a bit
	call	clockpulse	; pulse the clock line
	rrf	T0,F		; rotate next output bit into position
	decfsz	LOOPCOUNT,F	; repeat as required
	goto	ADCsetuploop
	;;
	;; ADC is now setup and is ready to output data
	;;
	bsf	STATUS,RP0	; select data bank 1
	bsf	ADC_DATA_TRIS		; change RB1 to input
	bcf	STATUS,RP0	; back to data bank 0
	clrf	T0		; set result to zero
	clrf	T1
	movlw	0x0d		; setup loop counter
	movwf	LOOPCOUNT
	call	ADCdelay	; extend sample time
	call	ADCdelay
ADCresultloop:
	bcf	STATUS,C	; clear carry for shift
	rlf	T1,F
	rlf	T0,F
	btfsc	ADC_DATA	; check result bit
	incf	T1,F
	call	clockpulse	; clock in next bit
	decfsz	LOOPCOUNT,F
	goto	ADCresultloop
	bsf	ADC_CS		; deactivate chip select to ADC
	
	return
;;
;; The ADC has finished sending the result. If we continue clocking it, it
;; will send the result again in reverse order. Thus we can do an error check
;; on everything but the LSB (which isn't repeated).
;;
;; The comparison will be done "in place" and no copy of the result will
;; be made. This could get slightly tricky....
;;
;; Because of the last clock pulse above, the second bit is already present.
;;
;;
;; NOTE: THIS IS CURRENTLY "DEAD CODE" and is NOT used.
;;
;; It DOES NOT WORK and still needs some debugging. But I found my problem
;; was somewhere else so I may never make this code work.
;;


	movlw	0x0b		; loop counter
	movwf	LOOPCOUNT
ADCcheckloop:
	bcf	STATUS,C		; clear carry for right shift
	btfsc	T1,0		; check LSB of result
	bsf		STATUS,C	; set carry if it is a one
	rrf		T0,F		; perform the shift
	rrf		T1,F		;
	btfsc	T1,0		; check current bit of result
	goto	ADCcheckforone
	btfsc	ADC_DATA	; do the check
	goto	repeatconversion	; the bits didn't match
	goto	bitchecked	; it was OK
ADCcheckforone:			; repeat for a one
	btfss	ADC_DATA
	goto	repeatconversion
bitchecked:
	call	clockpulse	; step to next bit
	decfsz	LOOPCOUNT,F
	goto	ADCcheckloop
;;
;; OK, the data checked out. Now we need to rotate the result a few more
;; bits to get it back to where it started.
;;
	movlw	0x05
	movwf	LOOPCOUNT
	bcf	STATUS,C		; clear carry for right shift
	btfsc	T1,0		; check LSB of result
	bsf		STATUS,C	; set carry if it is a one
ADCadjustloop:
	rrf		T0,F		; perform the shift
	rrf		T1,F		;
	decfsz	LOOPCOUNT,F
	goto ADCadjustloop
	bsf	ADC_CS		; deactivate chip select to ADC
	retlw	0
repeatconversion:
	bsf	ADC_CS		; deactivate chip select to ADC
	call	ADCdelay
	goto	startconversion
	end
