;
;********************************************************************
;                  Binary To BCD Conversion Routine
;      This routine converts a 16 Bit binary Number to a 5 Digit
; BCD Number. 
;
;       The 16 bit binary number is input in locations BARGB0 and
; BARGB1 with the high byte in BARGB0.
;       The 5 digit BCD number is returned in AARGB0, AARGB1, and AARGB2
; with AARGB0 containing the MSD in its right most nibble.
;
;   Performance :
;               Program Memory  :       35
;               Clock Cycles    :       885
;
;
;       Program:          B16TOBCD.ASM 
;       Revision Date:   
;                         1-13-97      Compatibility with MPASMWIN 1.40
;                         9-25-02      Convert to 16F628 and use AARG, BARG
;
;*******************************************************************;
;
;
	
;
	include         "p16f628.inc"
	include	data.inc

	global	B2_BCD
	code
;
B2_BCD  bcf     STATUS,C                ; clear the carry bit
	movlw   0x10
	movwf   LOOPCOUNT
	clrf    AARGB0
	clrf    AARGB1
	clrf    AARGB2
	
loop16  rlf     BARGB1, F
	rlf     BARGB0, F
	rlf     AARGB2, F
	rlf     AARGB1, F
	rlf     AARGB0, F
;
	decfsz  LOOPCOUNT, F
	goto    adjDEC
	RETLW   0
;
adjDEC  movlw   AARGB2
	movwf   FSR
	call    adjBCD
;
	movlw   AARGB1
	movwf   FSR
	call    adjBCD
;
	movlw   AARGB0
	movwf   FSR
	call    adjBCD
;
	goto    loop16
;
adjBCD  movlw   3
	addwf   INDF,W
	movwf   SIGN
	btfsc   SIGN,3          ; test if result > 7
	movwf   INDF
	movlw   0x30
	addwf   INDF,W
	movwf   SIGN 
	btfsc   SIGN,7          ; test if result > 7
	movwf   INDF               ; save as MSD
	RETLW   0
	END



