	include	p16f628.inc
	include data.inc

	global	cbeep_init, beep_off, beep_on, beep_long_pause
	global	beep_short_pause
	global	beep_start, beep_OK, beep_pause_1, beep_drogue
	global	beep_pause_2, beep_main_1, beep_pause_3, beep_main_2



TICKRATE equ	0x80

	code
	
	;; initialize Timer 2 and capture/compare module for buzzer
	;; duty cycle is 50% and frequency is 4KHz (more or less)
	;; with a 4MHz system clock. Timer 2 interupt is NOT used.
	;;
	;; Start with 1 second pause
	;; 

cbeep_init:
	movlw	0x07		; turn off analog comparators
	movwf	CMCON
	movlw	0x09		; setup PORTA
	movwf	PORTA

	bsf	STATUS,RP0
	movwf	TRISA
	movlw	0x3d
	bsf	STATUS,RP0
	movwf	PR2		; sets period
	bcf	STATUS,RP0
	movlw	0x1f		; half of PR2 value for 50% duty cycle
	movwf	CCPR1L
	movlw	0x05		; Timer 2 On, Prescale = 4
	movwf	T2CON
	movlw	0x2c		; set capture/compare module to PWM mode
	movwf	CCP1CON
	clrf	BEEP_STATE
	goto	beep_long_pause



beep_on:
	movlw	TICKRATE/10	; set beep time
	movwf	BEEP_TICK
	bsf	STATUS,RP0	; set to page 1
	bcf	TRISB,3		; turn on buzzer output
	bcf	STATUS,RP0	; back to page 0
	return

beep_long_pause:
	movlw	TICKRATE
	movwf	BEEP_TICK
	goto	beep_off
beep_short_pause:
	movlw	TICKRATE/5
	movwf	BEEP_TICK
	
beep_off:
	bsf	STATUS,RP0	; set to page 1
	bsf	TRISB,3		; turn off buzzer output
	bcf	STATUS,RP0	; back to page 0
	return


beep_start:
	;;
	;; 1 second has elapsed, reset BEEP_TICK, turn on buzzer, and
	;; set next state.
	;;
	incf	BEEP_STATE,F	; set next state
	goto	beep_on
	;;
	;; "OK" beep is finished. Start pause.
	;; 
beep_OK:
	incf	BEEP_STATE,F
	goto	beep_short_pause

	;;
	;; Pause after "OK" beep is done, now we have to figure out what to
	;; do next. First check for drogue continuity.
	;;
beep_pause_1:
	;; assume success
	incf	BEEP_STATE,F
	btfsc	PORTA,0		; zero is off
	goto	beep_on		; drogue is OK

beep_check_main:
	;; first setup assuming success
	incf	BEEP_STATE,F
	incf	BEEP_STATE,F
	;; then perform check
	btfsc	PORTA,3		; zero is off
	goto	beep_on
	;; nothing. goto state zero (big pause)
	clrf	BEEP_STATE
	goto	beep_long_pause
	;;
	;; Drogue beep complete
	;; 
beep_drogue:
	incf	BEEP_STATE,F
	goto	beep_short_pause

	;; pause after drogue beep complete. Now we check for the main.
	;; By being tricky, we can reuse some code.
beep_pause_2:	
	decf	BEEP_STATE,F
	goto	beep_check_main

	;; First beep for main complete. Pause.
beep_main_1:
	incf	BEEP_STATE,F
	goto	beep_short_pause

	;; pause done, now do second beep
	;; 
beep_pause_3:
	incf	BEEP_STATE,F
	goto	beep_on
	;;
	;; final beep complete. Start over.
	;; 
beep_main_2:
	clrf	BEEP_STATE
	goto	beep_long_pause
	

	end
