
	include p16f628.inc
	include data.inc
	include eedata.inc
	include adc.inc
	
	global	mainloop
	extern	KALMAN_STEP, state_machine, cbeep, beep_off, eeread
	extern	set_alt_gains
	extern	write_preflight, write_inflight, write_it_now, ee_end_preflight
	extern	beep_alt
	global	prelaunch, inflight, apogee, postapogee
	global	drogue, main, postmain, ground
	extern	get_altitude
	
LIFTOFFVELOCITY	equ	0x0f
LIFTOFFACCELERATION equ	0x0f
TICKRATE	equ	0x40
STATE_PRELAUNCH	equ	0
STATE_INFLIGHT	equ	1
STATE_APOGEE	equ	2
STATE_POSTAPOGEE	equ	3
STATE_DROGUE	equ	4
STATE_MAIN	equ	5
STATE_POSTMAIN	equ	6
STATE_GROUND	equ	7
			

	code
	;; Primary code loop. This is where all of the work gets done.
	;; Wait for timer 1 interupt to happen and then process a sample.
	;; Repeat forever.

mainloop:
 

	btfss	TFLAG,0
	goto	mainloop	; spin wheels madly doing nothing

	;;
	;; Interupt occured so we are off and running. First step is to
	;; clear the flag.
	bcf	TFLAG,0
	
	;;
	;; Perform a state update
	;;
	call	KALMAN_STEP
	;;
	;; Then let the state machine do its stuff
	;;
	call	state_machine

	goto	mainloop
	
	
prelaunch:	
	;; Code for pre-launch activity
	;;
	;; Run the buzzer code first.
	;;
	call	cbeep
	;;
	;; Then check to see if we need to advance to the inflight state
	;;
	btfsc	VELB0,7		; first check sign bit to verify positive
	goto	not_launched
	movf	VELB1,W		; get lower 8 bits of integer part of velocity
	addlw	-LIFTOFFVELOCITY	; compare with liftoff threshold
	btfss	STATUS,C	;  check for borrow
	goto	not_launched	; nope, not moving

	btfsc	ACCELB0,7	; We are moving, double check acceleration
	goto	not_launched	; negative acceleration
	movf	ACCELB0,W
	iorwf	ACCELB0,W	; see if MSB is non-zero
	btfss	STATUS,Z
	goto	launched	;  it was not zero
	movf	ACCELB1,W	; check LSB now
	addlw	-LIFTOFFACCELERATION
	btfsc	STATUS,C
	goto	launched
	;;
	;; OK, we are still awaiting launch. There are a few things to take
	;; care of. First update the long term average values for pressure
	;; and acceleration. This is done every four seconds and is a double
	;; buffered value. First copy the value we saved four seconds ago to
	;; another location. This is used as the resting value in all
	;; computations. Then save the current average.
	;;
	;; Values are 32 bits.
	;;
not_launched:	
	movf	TICKB2,W	; is lsB of timer ticks == 0?
	btfss	STATUS,Z
	goto	end_prelaunch	; it wasn't zero so we can go on.

	;;
	;; Do copy of acceleration
	;;
	bsf	STATUS,RP0	; switch to data page 1
	movf	AVE_ACCEL1B0,W
	movwf	AVE_ACCEL2B0
	movf	AVE_ACCEL1B1,W
	movwf	AVE_ACCEL2B1
	movf	AVE_ACCEL1B2,W
	movwf	AVE_ACCEL2B2
	movf	AVE_ACCEL1B3,W
	movwf	AVE_ACCEL2B3

	movf	AVE_ACCEL0B0,W
	movwf	AVE_ACCEL1B0
	movf	AVE_ACCEL0B1,W
	movwf	AVE_ACCEL1B1
	movf	AVE_ACCEL0B2,W
	movwf	AVE_ACCEL1B2
	movf	AVE_ACCEL0B3,W
	movwf	AVE_ACCEL1B3
	;;
	;; Repeat for pressure, except use filtered altitude since we
	;; don't need the actual counts like we do for acceleration
	;;
	movf	AVE_ALT1B0,W
	movwf	AVE_ALT2B0
	movf	AVE_ALT1B1,W
	movwf	AVE_ALT2B1
	movf	AVE_ALT1B2,W
	movwf	AVE_ALT2B2
	movf	AVE_ALT1B3,W
	movwf	AVE_ALT2B3
	;;
	;; This gets tedious since the data is on two different pages
	;;
	bcf	STATUS,RP0
	movf	ALTB0,W
	bsf	STATUS,RP0
	movwf	AVE_ALT1B0
	bcf	STATUS,RP0
	movf	ALTB1,W
	bsf	STATUS,RP0
	movwf	AVE_ALT1B1
	bcf	STATUS,RP0
	movf	ALTB2,W
	bsf	STATUS,RP0
	movwf	AVE_ALT1B2
	bcf	STATUS,RP0
	movf	ALTB3,W
	bsf	STATUS,RP0
	movwf	AVE_ALT1B3
	bcf	STATUS,RP0	; data page 0
end_prelaunch:
	call	write_preflight
	return
		
;;
;;	Launch has been detected. Change to the next sate, make sure the buzzer is
;;	turned off, write data to EEPROM, and then setup EEPROM for writing flight data.
;;
launched:
	movlw	STATE_INFLIGHT
	movwf	MAIN_STATE		; change state
	call	beep_off	; make sure buzzer is off
	call	write_preflight
	call	ee_end_preflight
	return

	;;
	;; Rocket is in boost/coast phase of flight. All we do here is
	;; wait for the velocity to turn negative. When it does, we do the
	;; apogee stuff.
inflight:

	btfss	VELB0,7		; check sign bit
	goto	end_inflight	; nope, still going up

;;	Velocity is now negative so this must be apogee.

	movlw	STATE_APOGEE	; update state
	movwf	MAIN_STATE
	bsf	PORTA,1		; turn on number 1 pyro output
	movlw	TICKRATE/10	; set a timer for one tenth second
	movwf	BEEP_TICK	; buzzer code isn't running so we can use this

	movf	ALTB0,W		; Copy peak altitude
	movwf	PALTB0
	movf	ALTB1,W
	movwf	PALTB1
	call	set_alt_gains	; switch Kalman filter gain settings
	call	write_it_now	; write data
	return
end_inflight:
	call	write_inflight
	return		; Thats it
	
	;;
	;; Apogee code. This just runs a timer to see when to shut off pyro
	;; output
	;; 
	;;
apogee:
	decfsz	BEEP_TICK,F
	goto	end_apogee	; still ticking
	bcf	PORTA,1		; turn off output
	movlw	STATE_DROGUE
	movwf	MAIN_STATE	; update state
	call	write_it_now
	return
end_apogee:	
	call	write_inflight
	return
	

postapogee:
	movlw	STATE_DROGUE	; we should never get here, but just in case...
	movwf	MAIN_STATE
	call	write_it_now
	return


drogue:

	;;
	;; Now descending under drogue parachute. Monitor altitude to check
	;; for main deployment.
	;;
	;; First subtract base altitude (in AVE_ALT2) from present altitude.
	;;
	movf	ALTB0,W		; get copy of altitude
	movwf	T0
	movf	ALTB1,W
	movwf	T1		; Only check the integer part

	bsf	STATUS,RP0	; data page 1
	movf	AVE_ALT2B1,W
	bcf	STATUS,RP0	; data page 0
	subwf	T1,F

	bsf	STATUS,RP0	; data page 1
	movf	AVE_ALT2B0,W
	bcf	STATUS,RP0	; data page 0
	btfss	STATUS,C
	addlw	1
	subwf	T0,F

	;;
	;; Now compare to selected deployment altitude, stored in EEPROM
	;;
	movlw	MAIN_ALT_HI
	call	eeread
	movwf	T2
	movlw	MAIN_ALT_LO
	call	eeread
	movwf	T3

	movf	T3,W
	subwf	T1,F

	movf	T2,W
	btfss	STATUS,C
	incfsz	T2,W
	subwf	T0,F		; if carry is clear, we are below main altitude

	btfsc	STATUS,C
	goto	end_drogue

	;; Time for the main

	bsf	PORTA,2			; turn on output
	movlw	TICKRATE/10	; set timer
	movwf	BEEP_TICK
	movlw	STATE_MAIN	; update state
	movwf	MAIN_STATE
	call	write_it_now
	return
end_drogue:
	call	write_inflight
	return 

main:
	decfsz	BEEP_TICK,F	; count down
	goto	end_main	; still ticking
	bcf	PORTA,2		; output off
	movlw	STATE_POSTMAIN
	movwf	MAIN_STATE
	call	write_it_now
	return
end_main:
	call	write_inflight
	return


;; After the main parachute event, the only thing left is the landing.
;; Once the rocket is on the ground (check for zero velocity), transition
;; to the on ground state and beep out max AGL altitude.

postmain:	
	movf	VELB0,W
	btfss	STATUS,Z
	goto	in_air
	movf	VELB1,W
	btfss	STATUS,Z
	goto	in_air

	movlw	STATE_GROUND
	movwf	MAIN_STATE
	call	write_it_now
	return

in_air:

	call	write_inflight
	return

	;; Only ground activity is to beep out altitude
	;; Once this starts, the main event loop is no longer used and
	;; is repeated forever.
ground:
	;; Compute AGL altitude by subtracting pre-launch altitude from
	;; peak altitude
	movf	PALTB0,W
	movwf	T0
	movf	PALTB1,W
	movwf	T1

	bsf	STATUS,RP0	; data page 1
	movf	AVE_ALT2B0,W
	bcf	STATUS,RP0	; data page 0
	movwf	T2
	bsf	STATUS,RP0	; data page 1
	movf	AVE_ALT2B1,W
	bcf	STATUS,RP0	; data page 0
	movwf	T3

	movf	T3,W
	subwf	T1,F

	movf	T2,W
	btfss	STATUS,C
	incfsz	T2,W
	subwf	T0,F		; subtraction complete
beep_forever:	
	;; Copy AGL altitude to BARG
	movf	T0,W
	movwf	BARGB0
	movf	T1,W
	movwf	BARGB1

	call	beep_alt
	goto	beep_forever

	end
