	include	p16f628.inc

	global	eeread, eewrite

	;; Provide default values for all EEPROM data
	;;
	;; Note:	 This table must match what is in the file
	;; data.inc so that the program can read the correct values.
	;;
	;; DO NOT change this without updating data.inc !!!!!!
	;;
	;; 
eedata:	code	0x2100

	;; Kalman gains
	;;
	;; 128 SPS, 1 meter, 0.25 meter/sec/sec, 20 to 1
	
	de	0x01, 0x79	; K1 (HI, LO)
	de	0x00, 0x83	; K2 (HI, LO)
	de	0x00, 0x00	; K3 (HI, LO)
	de	0x0, 0x0d	; K4 (HI, LO)
	de	0x01, 0x99	; K5 (HI, LO)
	de	0x33, 0x32	; K6 (HI, LO)
	
	;; Alternate Kalman gains. Pressure sensor only
	;;
	;; 128 SPS 1 meter, 1,000,000 to 1
	
	de	0x0d, 0xd1	; K1 (HI, LO)
	de	0x30, 0x85	; K2 (HI, LO)
	de	0x4b, 0x64	; K3 (HI, LO)

	;;
	;; Altitude above initial altitude for main parachute deployment
	;; 

	de	0x00, 0x80	; 128 meters

	;;
	;; Accelerometer sensitivity (meters/sec/sec/count) in 0.16 fixed point
	;; format
	;;

	de	0x51, 0xEE

        ;
        ;       Pressure to altitude conversion tables
        ;
        ;       Base altitude is a 16 bit integer
        ;
        ;       Slopes are 8.8 signed fixed point numbers
        ;
        ;       Base Pressure, high byte
        ;
        de      0x3c, 0x31, 0x28, 0x22
        de      0x1d, 0x18, 0x14, 0x10
        de      0x0d, 0x0a, 0x07, 0x04
        de      0x02, 0x00, 0xfd, 0xfb
        ;
        ;       Base Pressure, low byte
        ;
        de      0x53, 0x0d, 0xf4, 0x8a
        de      0x2f, 0x8f, 0x7a, 0xcf
        de      0x7a, 0x6a, 0x95, 0xf0
        de      0x76, 0x21, 0xed, 0xd5
        ;
        ;       Slope, high byte
        ;
        de      0xf4, 0xf7, 0xf9, 0xfa
        de      0xfb, 0xfb, 0xfc, 0xfc
        de      0xfc, 0xfd, 0xfd, 0xfd
        de      0xfd, 0xfd, 0xfd, 0xfe
        ;
        ;       Slope, low byte
        ;
		de	0xba, 0xe7, 0x96, 0xa5
		de	0x61, 0xeb, 0x56, 0xab
		de	0xf1, 0x2b, 0x5c, 0x86
		de	0xab, 0xcc, 0xe8, 0x02
	
	code	
	;;
	;; Routines to read and write the on-chip EEPROM data.
	;;


	;;
	;; eeread:
	;;	Call with address to read in W.
	;;	Data returned in W.
	;;
	;; Assumes data bank zero selected on entry.
	;; 
eeread:	bsf	STATUS,RP0	; data bank 1
	movwf	EEADR
	bsf	EECON1,RD
	movf	EEDATA,W	; get data;
	bcf	STATUS,RP0	; data bank 0
	return

	;;
	;; eewrite:
	;;	Set EEADR and EEDATA before calling.
	;;
	;; Assumes data bank one selected on entry.
	;;
	;; Calling program must set WREN
	;; 
eewrite:	
	bcf	INTCON,GIE	; disable interupts
	movlw	0x55
	movwf	EECON2
	movlw	0xaa
	movwf	EECON2
	bsf	EECON1,WR
	bsf	INTCON,GIE
eewait:	btfsc	EECON1,WR	; wait for write to complete
	goto	eewait
	return
	end
