	include p16f628.inc
	include data.inc
	
MSB	equ	7

	global	FXM3216S
	

	code



;	RCS Header $Id: fxm26.a16 2.3 1996/10/16 14:23:23 F.J.Testa Exp $

;	$Revision: 2.3 $

;       32x16 PIC16 FIXED POINT MULTIPLY ROUTINES
	;; extracted from Microchip apllication note AN617
;
;       Input:  fixed point arguments in AARG and BARG
;
;       Output: product AARGxBARG in AARG
;
;       All timings are worst case cycle counts
;
;       It is useful to note that the additional unsigned routines requiring a non-power of two
;       argument can be called in a signed multiply application where it is known that the
;       respective argument is nonnegative, thereby offering some improvement in
;       performance.
;
;         Routine            Clocks     Function
;
;       FXM3216S     423        32x16 -> 48 bit signed fixed point multiply
;       The above timings are based on the looped macros. If space permits,
;       approximately 65-88 clocks can be saved by using the unrolled macros.
;

;**********************************************************************************************
;**********************************************************************************************

;       32x16 Bit Multiplication Macros

SMUL3216L        macro

;       Max Timing:     2+13+6*26+25+2+6*27+26+7 = 393 clks

;       Min Timing:     2+7*6+5+2+6*6+5+6 = 98 clks

;       PM: 19+60 = 79            DM: 11


                MOVLW   0x8
                MOVWF   LOOPCOUNT

LOOPSM3216A
                RRF     BARGB1,F
                BTFSC  STATUS,C
                GOTO    ALSM3216NA
                DECFSZ  LOOPCOUNT,F
                GOTO    LOOPSM3216A

                MOVLW   0x7
                MOVWF   LOOPCOUNT

LOOPSM3216B
                RRF     BARGB0,F
                BTFSC  STATUS,C
                GOTO    BLSM3216NA
                DECFSZ  LOOPCOUNT,F
                GOTO    LOOPSM3216B

                CLRF    AARGB0
                CLRF    AARGB1
                CLRF    AARGB2
                CLRF    AARGB3
                RETLW   0x00

ALOOPSM3216
                RRF             BARGB1,F
                BTFSS  STATUS,C
                GOTO    ALSM3216NA
                MOVF   TEMPB3,W
                ADDWF   AARGB3,F
                MOVF            TEMPB2,W
                BTFSC          STATUS,C
                INCFSZ          TEMPB2,W
                ADDWF           AARGB2,F
                MOVF            TEMPB1,W
                BTFSC          STATUS,C
                INCFSZ          TEMPB1,W
                ADDWF           AARGB1,F
                MOVF            TEMPB0,W
                BTFSC          STATUS,C
                INCFSZ          TEMPB0,W
                ADDWF           AARGB0,F

ALSM3216NA      RLF    SIGN,W
                RRF    AARGB0,F
                RRF    AARGB1,F
                RRF    AARGB2,F
                RRF    AARGB3,F
                RRF    AARGB4,F
                DECFSZ  LOOPCOUNT,F
                GOTO    ALOOPSM3216

                MOVLW   0x7
                MOVWF   LOOPCOUNT

BLOOPSM3216
                RRF             BARGB0,F
                BTFSS  STATUS,C
                GOTO    BLSM3216NA
                MOVF   TEMPB3,W
                ADDWF   AARGB3,F
                MOVF            TEMPB2,W
                BTFSC          STATUS,C
                INCFSZ          TEMPB2,W
                ADDWF           AARGB2,F
                MOVF            TEMPB1,W
                BTFSC          STATUS,C
                INCFSZ          TEMPB1,W
                ADDWF           AARGB1,F
                MOVF            TEMPB0,W
                BTFSC          STATUS,C
                INCFSZ          TEMPB0,W
                ADDWF           AARGB0,F

BLSM3216NA      RLF    SIGN,W
                RRF    AARGB0,F
                RRF    AARGB1,F
                RRF    AARGB2,F
                RRF             AARGB3,F
                RRF             AARGB4,F
                RRF             AARGB5,F
                DECFSZ  LOOPCOUNT,F
                GOTO    BLOOPSM3216

                RLF    TEMPB0,W
                RRF    AARGB0,F
                RRF    AARGB1,F
                RRF    AARGB2,F
                RRF             AARGB3,F
                RRF             AARGB4,F
                RRF             AARGB5,F

                endm

;**********************************************************************************************
;**********************************************************************************************
        
;       32x16 Bit Signed Fixed Point Multiply 32x16 -> 32

;       Input:  16 bit signed fixed point multiplicand in AARGB0
;                       16 bit signed fixed point multiplier in BARGB0

;       Use:    CALL    FXM3216S

;       Output: 32 bit signed fixed point product in AARGB0

;       Result: AARG  <--  AARG x BARG

;       Max Timing:     13+393+2 = 408 clks                B > 0
;                       28+393+2 = 423 clks                B < 0

;       Min Timing:     13+98 = 111 clks

;       PM: 18+79+1 = 98              DM: 9


FXM3216S	CLRF	AARGB4          ; clear partial product
		CLRF	AARGB5
		CLRF	SIGN
		MOVF	AARGB0,W
		IORWF	AARGB1,W
		IORWF	AARGB2,W
		IORWF	AARGB3,W
		BTFSC	STATUS,Z
		RETLW	0x00
		
		MOVF	AARGB0,W
		XORWF	BARGB0,W
		MOVWF	TEMPB0
		BTFSC	TEMPB0,MSB
		COMF	SIGN,F

	        BTFSS   BARGB0,MSB
                GOTO    M3216SOK

                COMF            BARGB1,F
                COMF            BARGB0,F
                INCF            BARGB1,F
		BTFSC		STATUS,Z
		INCF		BARGB0,F

                COMF            AARGB3,F
                COMF            AARGB2,F
                COMF            AARGB1,F
                COMF            AARGB0,F
		INCF		AARGB3,F
		BTFSC		STATUS,Z
                INCF            AARGB2,F
                BTFSC          STATUS,Z
                INCF            AARGB1,F
		BTFSC		STATUS,Z
		INCF		AARGB0,F

		BTFSC	BARGB0,MSB
		GOTO	M3216SX

M3216SOK        MOVF   AARGB0,W
                MOVWF   TEMPB0
                MOVF   AARGB1,W
                MOVWF   TEMPB1
                MOVF   AARGB2,W
                MOVWF   TEMPB2
                MOVF   AARGB3,W
                MOVWF   TEMPB3

                SMUL3216L

                RETLW           0x00


M3216SX		CLRF	AARGB4
		CLRF	AARGB5
		RLF	SIGN,W
		RRF	AARGB0,F
		RRF	AARGB1,F
		RRF	AARGB2,F
		RRF	AARGB3,F
		RRF	AARGB4,F

		RETLW	0x00
	end
