	include p16f628.inc
	include data.inc

	global	cbeep
	global	state_machine, hex_to_ascii, end_interupt
	extern	kalman_init, init_ADC, init_serialee
	
	extern	mainloop
	extern	prelaunch, inflight, apogee, postapogee
	extern	drogue, main, postmain, ground
	extern	MAIN_STATE
	extern	cbeep_init
	extern	beep_start, beep_OK, beep_pause_1, beep_drogue
	extern	beep_pause_2, beep_main_1, beep_pause_3, beep_main_2
	extern	BEEP_STATE, BEEP_TICK
	extern	mother	


	;;
	;; This file contains the initialization code and other things
	;; that MUST be located in particular locations. Mainly state machine
	;; tables.
	

Reset	code	0x00		; locate at very start of memory

	goto	start		; This is the reset vector
	nop
	nop
	nop
	goto	interupt	; This is the interupt vector

Fixed	code	0x05
	;; What follows is the jump table for the main state machine. It
	;; must be located here to make sure it doesn't cross a memory
	;; page boundry.
	;;
	;; Jump to the appropriate section of code based
	;; on the current state.
	;;
	;; Currently, only a maximum of eight states are allowed. This can
	;; be expanded if required.
	;;

state_machine:
	movlw	high $
	movwf	PCLATH
	movf	MAIN_STATE,W	; get current state
	andlw	0x7		; maximum of 8 states
	addwf	PCL,F		; add to program counter
	goto	prelaunch
	goto	inflight
	goto	apogee
	goto	postapogee
	goto	drogue
	goto	main
	goto	postmain
	goto	ground



	;; Next is the state machine table for the continuity beep code.

	;;
	;; This code is used during the pre-flight phase to beep out the
	;; altimeters status. Main program loop should call this once per
	;; sample.
	;;
	;; One beep:	OK, no continuity.
	;; Two beeps:	OK, Drogue continuity only.
	;; Three beeps:	OK, Main continuity only.
	;; Four beeps:	OK, Drogue and Main continuity.
	;;
	;; The beeper uses the 16F628 PWM output which should be driving
	;; the buzzer.
	;;
	;; Timing:
	;;   Beep length:	0.1 second
	;;   between beeps:	0.2 seconds
	;;   between beep groups 1 second
	;;
	;; The code is organized as a state machine.
	;;
	;; Date space used:	2 bytes
	;;

cbeep:
	;; First decrement counter and check for zero
	
	decfsz	BEEP_TICK,F	; decrement counter
	return			; non-zero, done
	;;
	;; Counter is zero. We need to do something. We use the
	;; current state and a lookup table to figure out what.
	;; 
	movlw	high $
	movwf	PCLATH
	movf	BEEP_STATE,W	; get current state
	andlw	0x7		; maximum of 8 states
	addwf	PCL,F		; add to program counter
	goto	beep_start
	goto	beep_OK
	goto	beep_pause_1
	goto	beep_drogue
	goto	beep_pause_2
	goto	beep_main_1
	goto	beep_pause_3
	goto	beep_main_2



hex_to_ascii:
	andlw	0x0f
	addwf	PCL,F
	retlw	'0'
	retlw	'1'
	retlw	'2'
	retlw	'3'
	retlw	'4'
	retlw	'5'
	retlw	'6'
	retlw	'7'
	retlw	'8'
	retlw	'9'
	retlw	'A'
	retlw	'B'
	retlw	'C'
	retlw	'D'
	retlw	'E'
	retlw	'F'



	code

	;; Initialization for power on reset.
	;; Setup all ports, timers, interupts, etc.
	
start:
	clrf	PCLATH		; paranoia
	bcf	STATUS,RP0
	bcf	STATUS,RP1
	;; 
	;; Setup Timer 1 to overflow every 1/128 second
	;;
	;; Assumes a 4MHz clock
	;;
	
	movlw	high (D'65536' - D'7812')
	movwf	TMR1H
	movlw	low  (D'65536' - D'7812')
	movwf	TMR1L
	movlw	0x01
	movwf	T1CON		; prescale = 1, Timer 1 on
	bsf	STATUS,RP0
	bsf	PIE1,0
	bcf	STATUS,RP0
	bsf	INTCON,PEIE
	bsf	INTCON,GIE

	call	mother		; see if host PC is attached

	call	init_serialee	; startup serial EEPROM link
	
	call	init_ADC	; setup ADC

	call	cbeep_init	; setup buzzer stuff

	call	kalman_init	; initialize state variables and such

	clrf	MAIN_STATE	; start in pre-launch
	goto	mainloop	; start it up




	
	;; Interupt handler
	;;

interupt:
	movwf	W_TEMP
	swapf	STATUS,W
	bcf	STATUS,RP0
	bcf	STATUS,RP1	; make sure on data page 0
	
	movwf	STATUS_TEMP
	btfsc	PIR1,TMR1IF	; check for Timer 1 interupt
	goto	timer1int

	;;
	;; If we get here, we are in serious trouble. An interupt
	;; has arrived that we have no code to handle.
	;;
	;; Punt
	
	goto	end_interupt
	
timer1int:	
	bcf	T1CON,0		; Turn off Timer1 for a moment so we can reload
	movlw	high (D'65536' - D'7812')
	movwf	TMR1H
	movlw	low  (D'65536' - D'7812')
	movwf	TMR1L
	bcf	PIR1,TMR1IF	; Reset Timer 1 interupt flag
	bsf	T1CON,0		; Turn it back on


	incf	TICKB2,F	; then increment timer tick counter
	btfsc	STATUS,C
	incf	TICKB1,F
	btfsc	STATUS,C
	incf	TICKB0,F

	bsf	TFLAG,0			; set timer flag

end_interupt:
	bcf	STATUS,RP0
	bcf	STATUS,RP1
	swapf	STATUS_TEMP,W
	movwf	STATUS
	swapf	W_TEMP,F
	swapf	W_TEMP,W
	retfie


	end
