/*
Compute piecewise linear data tables for altimeter.


Assumptions:

12 bit ADC with 0 to 5 volt input range
45.9mV/kPa sensor sensitivity

14.36 kPa offset (swagged)

*/

#include <stdio.h>
#include <math.h>

#define OFFSET    10.721         /* kPa    */
#define SENSITIVITY       45.0           /* mv/kPa */

main(int argc, char **argv)
{
  double altitude[20];
  double slopes[20];
  double base[20];
  double pressure;

  int i, j, k;

  j = 4096;
  for( i = 16; i >= 0; i--)
    {
      pressure = ((double)(j)/4096.0)*5000.0/SENSITIVITY + OFFSET;
      altitude[i] = (288.14- 288.08 * pow(pressure/101.29, 1/5.256))/0.00649;
      j -= 256;
      if(j < 0)
	break;
    }
  for(i = 0; i < 16; i++)
    {
      slopes[i] = (altitude[i+1] - altitude[i])/256.0;
    }
  printf("Altitude, slope\n");
  for(i = 0; i < 16; i++)
    {
      printf("%f %f\n", altitude[i], slopes[i]);
    }
  printf("\n\nData suitable for MPASM\n");

  printf("\t;\n\t;\tPressure to altitude conversion tables\n\t;\n");
  printf("\t;\tBase altitude is a 16 bit integer\n\t;\n");
  printf("\t;\tSlopes are 8.8 signed fixed point numbers\n\t;\n");

  printf("\t;\tBase Pressure, high byte\n\t;\n");
  for(i = 0; i < 16; i+=4)
    {
      printf("\tde\t%#04x, %#04x, %#04x, %#04x\n", ((int)altitude[i]) >> 8,
	     ((int)altitude[i+1]) >> 8,
	     ((int)altitude[i+2]) >> 8,
	     ((int)altitude[i+3]) >> 8 & 0xff);
    }
  printf("\t;\n\t;\tBase Pressure, low byte\n\t;\n");
  for(i = 0; i < 16; i+=4)
    {
      printf("\tde\t%#04x, %#04x, %#04x, %#04x\n", ((int)altitude[i]) & 0xff,
	     ((int)altitude[i+1]) & 0xff,
	     ((int)altitude[i+2]) & 0xff,
	     ((int)altitude[i+3]) & 0xff);
    }
  printf("\t;\n\t;\tSlope, high byte\n\t;\n");
  for(i = 0; i < 16; i+=4)
    {
      printf("\tde\t%#04x, %#04x, %#04x, %#04x\n", ((int)(slopes[i]*256) >> 8) & 0xff,
	     ((int)(slopes[i+1]*256) >> 8) & 0xff,
	     ((int)(slopes[i+2]*256) >> 8) & 0xff,
	     ((int)(slopes[i+3]*256) >> 8) & 0xff);
    }
  printf("\t;\n\t;\tSlope, low byte\n\t;\n");
  for(i = 0; i < 16; i+=4)
    {
      printf("\tde\t%#04x, %#04x, %#04x, %#04x\n",
	     ((int)(slopes[i]*256)) & 0xff,
	     (int)(slopes[i+1]*256) & 0xff,
	     (int)(slopes[i+2]*256) & 0xff,
	     (int)(slopes[i+3]*256) & 0xff);
    }


}
